#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: out2golden.sh,v $
#	$Revision: 1.7 $
#	$Date: 2008/06/10 10:34:03 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script is used for substituting the *.golden files data
#	with the content of corresponding *.out files
#

#
# This function recursively traverses the folders, using the test_list
# files and on the bottom level, i.e. the level of tests, it renames
# *.out files into *.golden.
# @param ${1} the name of the current test suite or test
#             (the directory we have descended into)
#
doOutToGoldenRecursive () {
	#Allows to trace the progress
	echo -n "${1}/"	
	
	#Go down if there is a test_list file here, i.e. we are on a test-suite level
	if test -f "test_list"; then
		#Go through the test-suite tests or sub-suites
		TEST_SUITES=`eval cat test_list | sed -e '/#.*/d'`
		for test_suite in ${TEST_SUITES}; do
			#Move in
			cd ${test_suite}
			#Call recursive cleaning
			doOutToGoldenRecursive ${test_suite}
			#Move out
			cd ..
		done
	else 
		#We are on the test-case level, it is time to do the moving.
		if test -f ${1}.out; then
			rm -f ${1}.golden
			mv ${1}.out ${1}.golden
			rm -f ${1}.diff
		fi
	fi
	
	#Erase the "${1}/" string from the output
	NAME_LENGTH=${#1}
	BSP_STRING=""
        BLANK_STRING=""
	let "NUMBER_OF_BSP = ${NAME_LENGTH} + 1"
	for (( i=1 ; i <= ${NUMBER_OF_BSP} ; i++ )) ; do
		BSP_STRING="${BSP_STRING}\b"
		BLANK_STRING="${BLANK_STRING} "
	done
	echo -n -e "${BSP_STRING}${BLANK_STRING}${BSP_STRING}"
}

#Make .out files into *.golden
echo -n "Processing: "
doOutToGoldenRecursive ""
echo "DONE"
