#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: test.sh,v $
#	$Revision: 1.7 $
#	$Date: 2008/06/13 06:56:29 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script runs tests
#

export PATH=.:${PATH}		#Add local directory to the path, otherwise the tests will not run

export TEST_SUITE_ROOT_PATH=`eval pwd`

#
# This function recursively traverses the folders, using the test_list
# files and and invokes all the tests
# @param ${1} the folder we descended from (long name)
# @param ${2} the folder we descended to (short name)
# @param ${3} should be "1" for the invocation on the first test_list entry
#
doRecursiveTesting () {
	#Go down if there is a test_list file here, i.e. we are on a test-suite level
	if test -f "test_list"; then
		#Go through the test-suite tests or sub-suites
		TEST_SUITES=`eval cat test_list | sed -e '/#.*/d'`
		INDEX="1"
		for test_suite in ${TEST_SUITES}; do
			#Move in
			cd ${test_suite}
			#Call recursive testing
			doRecursiveTesting "${1}/${2}" ${test_suite} ${INDEX}
			#Move out
			cd ..
			#Increase the test_list entry index
			let "INDEX = ${INDEX} + 1"
		done
	else
		#We are in the test folder, print the outer folder name if it is the first test
		if test \( "${3}" -eq "1" \); then
			echo "${THE_SYMBOL_LINE}"
			echo -e "${THE_LINE_SYMBOL} ${1}:"
		fi
		
		#Invoke the test and process its output
		export TEST_NAME="${2}"
		#Construct a long test name for the valgrind log-file name
		export LONG_TEST_NAME=`eval echo "${1}/${2}" | sed -e 's/\.\/b\///g' | sed -e 's/\//./g'`
		echo -e -n "\t${TEST_NAME}"
		${TEST_NAME}.sh
		${TEST_SUITE_ROOT_PATH}/pf.sh ${TEST_NAME}
	fi
}

#Call test function
doRecursiveTesting "." "\b" "0"

