/**
*	WARNING: Do Not Remove This Section
*
*	$RCSfile: test_bitset_01.c,v $
*	$Revision: 1.2 $
*	$Date: 2007/11/29 10:58:45 $
*	$Author: i.zapreev $
*	
*	MRMC is a model checker for discrete-time and continuous-time Markov
*	reward models. It supports reward extensions of PCTL and CSL (PRCTL 
*	and CSRL), and allows for the automated verification of properties
*	concerning long-run and instantaneous rewards as well as cumulative
*	rewards.
*	
* 	Copyright (C) The University of Twente, 2004-2008.
* 	Copyright (C) RWTH Aachen, 2008-2009.
*	Authors: Joost-Pieter Katoen, Maneesh Khattri, Ivan Zapreev, Tim Kemna
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Main contact:
*		Lehrstuhl für Informatik 2, RWTH Aachen University
*		Ahornstrasse 55, 52074 Aachen, Germany
*		E-mail: info@mrmc-tool.org
*	
*	Old ontact:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: This is for testing the bitset only.
*	Uses: Definition of bitset - bitset.h
*/

# include "macro.h"
# include "bitset.h"

/*
NOTE: THE TESTS IN THIS FILE SHOULD NOT FAIL. IF ONE OF THE
TESTS FAILS - SOMETHING'S REALLY WRONG.

IN CONTRAST TO THE TESTS HERE IN PRACTICE SOME INVOCATIONS
CAN FAIL DUE TO SPURIOUS INPUTS PROVIDED THAT THE TESTS HERE
WERE SUCCESSFUL. CONSEQUENTLY IT IS NECESSARY TO CHECK FOR
SPURIOUS INPUTS IN PRACTICAL SCENARIOS.
*/

int main(){
	const state_count initial_size = 16;
	const state_count new_size=initial_size+(state_count)BITSET_BLOCK_SIZE;
	state_index curr_idx = state_index_NONE;
	
	/* Allocate, fill with ones and print */
	/*@null@*/ /*@only@*/ bitset * pBitSet = get_new_bitset(initial_size);
	if ( (bitset *) NULL == pBitSet )
		err_msg_0(err_CALLBY, "main()", EXIT_FAILURE);
	if ( err_state_iserror(fill_bitset_one(pBitSet))
			|| err_state_iserror(print_bitset_states(pBitSet)) )
		err_msg_0(err_CALLBY, "main()",
				(free_bitset(pBitSet), EXIT_FAILURE));
	printf("\n");
	
	while ( (curr_idx=get_idx_next_non_zero(pBitSet, curr_idx))
				!= state_index_NONE )
	{
		if ( state_index_ERROR == curr_idx )
			err_msg_0(err_CALLBY, "main()",
				(free_bitset(pBitSet), EXIT_FAILURE));
		printf("%d, ", curr_idx+1 );
	}
	printf("\n");
	
	/* Extend bitset such that it is filled with ones */
	pBitSet = bs_extend(pBitSet, new_size, TRUE);
	if ( (bitset *) NULL == pBitSet )
		err_msg_0(err_CALLBY, "main()", EXIT_FAILURE);
	if ( err_state_iserror(print_bitset_states(pBitSet)) )
		err_msg_0(err_CALLBY, "main()",
				(free_bitset(pBitSet), EXIT_FAILURE));
	printf("\n");
	
	curr_idx = state_index_NONE;
	while ( (curr_idx = get_idx_next_non_zero(pBitSet, curr_idx))
				!= state_index_NONE )
	{
		if ( state_index_ERROR == curr_idx )
			err_msg_0(err_CALLBY, "main()",
				(free_bitset(pBitSet), EXIT_FAILURE));
		printf("%d, ", curr_idx+1 );
	}
	printf("\n");
	free_bitset(pBitSet);
	
	return 0;
}
