#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: compare_out_golden_prob.awk,v $
#	$Revision: 1.5 $
#	$Date: 2007/06/11 08:17:39 $
#	$Author: zapreevis $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2007.
#	Authors: Christina Jansen, Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: The input are the files, which should be numerically
#	compared concerning a given error bound. The difference mustn't be bigger
#	than twice the error bound. Files should contain equal number of results
#	to compare.
#
#	@param ${1} the error bound e.g. 1.0e-6
#	@param ${2} the output file of the result vector, contains only one row with numbers to compare
#	@param ${3} the golden file of the result vector, contains only one row with expected numbers
#
#	NOTE: This script is called from compare_out_golden_prob.sh

BEGIN {
	RS="\n | ";
	input_data_files = 0;
	actual_error_bound = 2*error_bound;
	if( error_bound != ""){
		printf("      error bound: +/- %1.10g\n", actual_error_bound);
	}else{
		exit		
	}
}

{
	if( $0 != "" ){
		#number of results to compare
		numbers[input_data_files] = NF;
		printf("      Reading data file:   %i \n", input_data_files);
		
		#Store input in data array
		for(field = 0; field < NF; field++){
			data[input_data_files, field] = $(field+1);
		}
		input_data_files++;
		close($0)
	}
}

END {
	#progress only if files have equal number of results to compare
	if( numbers[0] == numbers[1] ){
		max_fail_index = -1;
		#calculate difference of result pair and compare to the given error bound
		for( field = 0; field < numbers[1]; field++ ){
			fail_difference = abs(data[0,field]-data[1,field]);
			if ( fail_difference > actual_error_bound ){
				if( max_fail_difference < fail_difference ){
					max_fail_difference = fail_difference;
					max_fail_index = field;
				}
			}
		}
		if( max_fail_index == -1 ){
			printf("      Probabilitity of state 1 to %i : ....PASS\n", field +1);
		}
		else{
			printf("      Max probabilitity-failure state %i : ....FAIL\n      Max difference: %1.10g\n", max_fail_index+1, max_fail_difference);
		}
			
	}else{
		print " ERROR: Wrong input...Files differ in length \n";
	}
}

#calculates |x|
function abs(x){
  return x < 0 ? -x : x
}
