#!/bin/bash -u
#../../../scripts/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: test_suite.sh,v $
#	$Revision: 1.31 $
#	$Date: 2008/06/13 07:38:18 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script runs tests
#

#Include the waiting and statistics functions
. ${PERFORMANCE_SHELL_SCRIPTS}/on_wait_statistics_gathering.sh

TESTS=`eval cat ${1} | sed -e '/#.*/d'`

INPUTS=`eval cat ${2} | sed -e '/#.*/d'`

##########################################
#%%%%%%%%%%%%EXECUTE TESTS%%%%%%%%%%%%%%%#
##########################################

export PATH=.:${PATH}		#Add local directory to the path, otherwise the tests will not run

############################################
#Perform tests, with all given input files,#
#   for the given number of repetitions,   #
#         with all given options.          #
############################################

#Run
for test in ${TESTS}
do
	#Set TEST_NAME
	export TEST_NAME=${test}
	#Set the input files for the given test
	export INPUT_DATA_FILES=`eval source input.data.files`
	
	cd ${test}
	echo "    ${test}: "
	
	#Generate the model and initialize
	echo -n "      Generating the model .........."
	rm -f ${TEST_NAME}.prism.log.out
	${TEST_NAME}.sh 1>${TEST_NAME}.prism.log.out
	echo "DONE"
	
	########################################################
	#    Do functional part of the performance testing,    #
	#just to make sure that MRMC still functions correctly.#
	########################################################

	#Run PERFORMANCE TESTS in the functional mode first
	#In order to test that MRMC works fine
	echo "        Functional test:"
	for input in ${INPUTS}
	do
		input_file=${input}.input
		echo "          ${input}:"
		#Run MRMC with different options
		cat ../option_list | sed -e '/#.*/d' | while read options; do
			#Export the test-case name.
			OPTS=`eval echo ${options} | sed -e 's/ /./'`
			export TEST_CASE_NAME=${TEST_NAME}.${OPTS}.${input}
			#We have to export it here for valgrind, since LONG_TEST_CASE_NAME
			#is used for forming the log file name.
			export LONG_TEST_CASE_NAME=${TEST_SUITE_NAME}.${TEST_SUBSUITE_NAME}.${TEST_CASE_NAME}
			
			echo -n "            ${TEST_CASE_NAME}: "
			
			#Clean the old files if any
			rm -f ${TEST_CASE_NAME}.memstat ${TEST_CASE_NAME}.out
			
			#Run the test
			${MRMC} ${options} ${INPUT_DATA_FILES} < ../${input_file} 2>&1 > ${TEST_CASE_NAME}.out &
			
			#Collect the PID of the process
			PID=$!
			
			#Do the waiting with the timeout
			doWait "${PERFORMANCE_TEST_TIMEOUT_SECS}" "${PID}" "${TEST_CASE_NAME}.memstat"
			
			#cat "" >> ${TEST_CASE_NAME}.golden
			#Run the functional testing like script to tests for PASS/FAIL
			${LUMPING_SHELL_SCRIPTS}/pf.sh ${TEST_CASE_NAME}
		done
	done

	#Rerun PERFORMANCE TESTS with different input files
	echo "        Performance test:"
	for input in ${INPUTS}
	do
		input_file=${input}.input
		echo -n "          ${input}:"
		#Rerun the experiments the required number of times
		for (( i=0 ; i < ${NUMBER_OF_PERFORMANCE_REPETITIONS} ; i=i+1 )) ; do
			echo -n " ${i}:"
			#Run MRMC with different options
			cat ../option_list | sed -e '/#.*/d' | while read options; do
				OPTS=`eval echo ${options} | sed -e 's/ /./g'`
				#We have to export it here for valgrind, since TEST_CASE_NAME
				#is used for forming the log file name.
				export TEST_CASE_NAME=${TEST_NAME}.${OPTS}.${input}
				
				#Clean up the tmp files if any
				rm -f ${TEST_CASE_NAME}.tmp
				
				#WARNING 'Elapsed' here is used to sort out the line
				#with the Elapsed time. It should be put into some
				#config file, since the MRMC output may change!
				${MRMC} ${options} ${INPUT_DATA_FILES} < ../${input_file} > ${TEST_CASE_NAME}.tmp &
				
				#Collect the PID of the process
				PID=$!
				
				#Do the waiting with the timeout
				doWait "${PERFORMANCE_TEST_TIMEOUT_SECS}" "${PID}" ""
				
				#WARNING: THERE HAS TO BE NO OPERATORS BETWEEN doWait and the next if!
				
				#Test if the test run ended with a time out.
				if test $? -eq 1; then
					#If YES, Produce the dummy statistics for the used tool
					echo "Timeout 1: ${TIME_OUT_MILLISEC} milli sec(s)" >> ${TEST_CASE_NAME}.results
					echo "Timeout 2: ${TIME_OUT_MILLISEC} milli sec(s)" >> ${TEST_CASE_NAME}.results
				else
					#Get the elapsed time results
					cat ${TEST_CASE_NAME}.tmp | grep Elapsed >> ${TEST_CASE_NAME}.results
				fi
				rm -f ${TEST_CASE_NAME}.tmp
			done
		done
		echo " DONE"
	done
	
	#Remove the tra files and de-initialize
	rm -f  ${INPUT_DATA_FILES}
	#Move to the test suite directory
	cd ..
done

if test ${NUMBER_OF_PERFORMANCE_REPETITIONS} -ne  0; then
	#Perform data collection and processing for the performance statistics.
	${LUMPING_SHELL_SCRIPTS}/statistics.sh ${1} ${2} performance
fi

#Perform data collection and processing for the memory statistics.
${LUMPING_SHELL_SCRIPTS}/statistics.sh ${1} ${2} memory
