#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: on_the_fly_average.awk,v $
#	$Revision: 1.1 $
#	$Date: 2007/12/07 11:36:18 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script allows to vcompute on-the-fly averages:
#
#	If we do not know the number of measurements we will have to do in advance,
#	and we may get overflow if we try to first sum up all the "VSZ RSS" pairs
#	and then divide the result by the total number of measurements.
#	That is why we use recursive formula, which might suffer from round orr
#	errors though. For the sequence of measurements:
#		c_{1}, c_{2}, c_{3}, ... c_{n-1}, c_{n}, ...
#	we define the on-the-fly average on each iteration as:
#	A_{1} = c_{1}
#	A_{n} = A_{n-1} * ( n-1 ) / n + c_{n} / n
#
#	@param old_average is A_{n-1}
#	@param new_value is c_{n}
#	@param iteration is n

BEGIN {
	#Compute A_{n}
	if ( iteration >= 1 ) {
		result = old_average * ( iteration - 1 ) / iteration + new_value / iteration;
		printf("%1.7f", result);
	}else{
		print "------------ on_the_fly_average.awk ------------" > "/dev/stderr";
		print "ERROR: The iteration number is less than 1!" > "/dev/stderr";
		print "USAGE: A_{n} = A_{n-1} * ( n-1 ) / n + c_{n} / n" > "/dev/stderr";
		print "\t old_average is A_{n-1}" > "/dev/stderr";
		print "\t new_value is c_{n}" > "/dev/stderr";
		print "\t iteration is n" > "/dev/stderr";
	}
}
