/**
*	WARNING: Do Not Remove This Section
*
*	$RCSfile: convert_tra_uniformized.c,v $
*	$Revision: 1.3 $
*	$Date: 2007/12/04 22:46:38 $
*	$Author: i.zapreev $
*	
*	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
*	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
*	automated verification of properties concerning long-run and instantaneous rewards
*	as well as cumulative rewards.
*	
*	Copyright (C) The University of Twente, 2004-2006.
*	Authors: Ivan Zapreev
*
*	This program is free software; you can redistribute it and/or
*	modify it under the terms of the GNU General Public License
*	as published by the Free Software Foundation; either version 2
*	of the License, or (at your option) any later version.
*
*	This program is distributed in the hope that it will be useful,
*	but WITHOUT ANY WARRANTY; without even the implied warranty of
*	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*	GNU General Public License for more details.
*
*	You should have received a copy of the GNU General Public License
*	along with this program; if not, write to the Free Software
*	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*
*	Our contacts:
*		Formal Methods and Tools Group, University of Twente,
*		P.O. Box 217, 7500 AE Enschede, The Netherlands,
*		Phone: +31 53 4893767, Fax: +31 53 4893247,
*		E-mail: mrmc@cs.utwente.nl
*
*	Source description: Convert a CTMC to its uniformised DTMC, both in .tra format.
*	
*/

#include <stdio.h>
#include <stdlib.h>

int read_tra_file(char *infile, char *outfile)
{
	FILE *ip = NULL, *op = NULL;
	char  s[1024];
	int size = 0, row = 0, col = 0, nnz = 0, new_self_loops = 0;
	double val = 0.0;
	double *e = NULL, *diag = NULL, u, q = 0.0;
	
	/* determine row sums */
	ip = fopen(infile, "r");
	if( ip==NULL ) return 0;
	if(fgets( s, 1024, ip )!=NULL)
	{
		sscanf( s, "STATES %d\n", &size);
		new_self_loops = size;
		e = (double *) calloc(size, sizeof(double));
		diag = (double *) calloc(size, sizeof(double));
		if(fgets( s, 1024, ip )!=NULL)
		{
			sscanf( s, "TRANSITIONS %d\n", &nnz);
			while (NULL != fgets( s, 1024, ip ))
			{
				sscanf( s, "%d %d %lg\n", &row, &col, &val );
				e[row-1] += val;
				if(e[row-1] > q){
					q = e[row-1];
				}
				if(row == col){
					diag[row-1] = val;
					new_self_loops--;
				}
			}
		}
	}
	(void)fclose(ip);
	
	q += 1.0; /* make q strictly larger */
	printf("q = %e\n", q);
	
	/* print uniformised DTMC */
	ip = fopen(infile, "r");
	op = fopen(outfile, "w");
	if(ip==NULL||op==NULL) return 0;
	if(fgets( s, 1024, ip )!=NULL)
	{
		sscanf( s, "STATES %d\n", &size);
		fprintf(op, "STATES %d\n", size);
		if(fgets( s, 1024, ip )!=NULL)
		{
			sscanf( s, "TRANSITIONS %d\n", &nnz);
			fprintf(op, "TRANSITIONS %d\n", nnz + new_self_loops);
			while (NULL != fgets( s, 1024, ip ))
			{
				sscanf( s, "%d %d %lg\n", &row, &col, &val );
				
				if(row == col){
					u = (val - e[row-1])/q + 1.0;
				}else{
					if(diag[row-1] == 0.0){
						/* if this state has no self-loop, we add it now */
						/* adding self-loops may not preserve ordering of row values, but this is no problem for MRMC */
						u = (0.0 - e[row-1])/q + 1.0;
						fprintf(op, "%d %d %e\n", row, row, u);
						diag[row-1] = u;
					}
					u = val/q;
				}
				fprintf(op, "%d %d %e\n", row, col, u);
			}
		}
	}
	(void)fclose(ip);
	(void)fclose(op);
	
	free(e);
	free(diag);
	
	return 1;
}

int main(int argc, char **argv)
{
	if(argc==3)
		printf("done: %d\n",read_tra_file(argv[1], argv[2]));
	else
		printf("FORMAT: <infile> <outfile> \n");
	return 0;
}
