#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: out2golden.sh,v $
#	$Revision: 1.1 $
#	$Date: 2007/12/07 17:27:59 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script moves out or output files of the test suite
#   into the golden files, including putting them to the SVN.
#
#	@param ${1} the test_list file
#
#	NOTE: The script is supposed to be invoked from the test suite directory.
#	For example from:
#		MRMC/test/performance_tests/lumping/dtmrm_lumping/rme 
#

TESTS=`eval cat ${1} | sed -e '/#.*/d'` ; export TESTS

#%%%%%%%%%%%%EXECUTE TESTS%%%%%%%%%%%%%%%
export PATH=.:${PATH}		#Add local directory to the path, otherwise the tests will not run

echo "---Move output files into the golden files---"
WORK_TO_DO="0"
for test in ${TESTS}
do
	echo "Processing the test: ${test}"
	cd ${test}
	
	#Remove Prism output files first
	rm -f *.prism.log.out
	
	RENAME_ZIP="0"
	#Process all goldens in the directory
	OUT_OLD_FILES=`eval ls *.out 2>/dev/null`
	OUT_NEW_FILES=`eval ls *.output 2>/dev/null`
	OUT_FILES=""
	if test \( -n "${OUT_OLD_FILES}" \) -a \( -n "${OUT_NEW_FILES}" \); then
		#If somehow we have both .out and .output files then we can not decide which we need
		echo "ERROR: We have both *.out and *.output files present, please remove the ones"
		echo "which should not be converted to golden files and re-run the script!"
		exit
	else
		OUT_FILES=${OUT_OLD_FILES}${OUT_NEW_FILES}
	fi
	
	for out in ${OUT_FILES}; do
		TEST_CASE_NAME=`echo ${out} | sed -e 's/.output//' | sed -e 's/.out//'`
		echo "  Processing the test case:"
		echo -n "    ${TEST_CASE_NAME} .........."
		
		#Remove goldens if any
		rm -f ${TEST_CASE_NAME}.golden
		
		#Move out to golden
		#If there is just one type of files present then we used them
		if test -n "${OUT_OLD_FILES}"; then
			mv ${TEST_CASE_NAME}.out ${TEST_CASE_NAME}.golden
		else 
			mv ${TEST_CASE_NAME}.output ${TEST_CASE_NAME}.golden
		fi
		rm -f ${TEST_CASE_NAME}.diff
		
		#Pack new goldens back
		zip -q ${test}.zip -u ${TEST_CASE_NAME}.golden 2>&1 1> /dev/null
		rm -f ${TEST_CASE_NAME}.golden
		
		#Mark the fact that we have to rename .zip in order
		#to replace and old one with the new one in SVN
		RENAME_ZIP="1"
		#Mark the fact that we will have to do SVN job later
		WORK_TO_DO="1"
		
		echo "DONE"
	done
	
	#Rename the zip file
	if test "${RENAME_ZIP}" != "0"; then
		mv ${test}.zip ${test}.zip.new
	fi
	
	cd ..
done

if test "${WORK_TO_DO}" != "0"; then
	echo "---Remove old versions of golden .zip files from the server---"
	#Now remove the old version from SVN, note the password will be aquired
	echo "* Scheduling old files for removal."
	svn remove 2>&1 | grep "for removal"
	#Commit removal
	echo "* Removing the files old files."
	svn commit -m "Removing the old version of the golden files" 2>&1 | grep "<--" | sed -e "s/.*.zip,v//g"
	
	echo "---Name the new golden .zip files---"
	#Rename .zip files back
	GOLDEN_UPDATE=""
	for test in ${TESTS}
	do
		echo -n "Processing the test: ${test} .........."
		cd ${test}
		
		if test -f ${test}.zip.new; then
			mv ${test}.zip.new ${test}.zip
			GOLDEN_UPDATE="${GOLDEN_UPDATE} ${test}/${test}.zip"
		fi
		
		echo "DONE"
		cd ..
	done
	
	echo "---Add new versions of golden .zip files to the server---"
	#Add golden files back
	if test "${GOLDEN_UPDATE}" != ""; then
		echo "* Scheduling new files for adding."
		svn add ${GOLDEN_UPDATE} 2>&1 | grep "after dead revision"
	fi
	#Commit removal
	echo "* Adding the new files."
	svn commit -m "Adding the new version of the golden files" 2>&1 | grep "<--" | sed -e "s/.*.zip,v//g"
else
	echo "WARNING: There is nothing to be done."
fi
echo "---WE ARE DONE---"
