#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: pf.sh,v $
#	$Revision: 1.3 $
#	$Date: 2007/12/09 21:32:27 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: Compare two files in an IF statement, checking the return status of DIFF.
#		We are interested in is the exit status of DIFF.
#
#	@patam ${1} is the test name which we currently execute.
#	@param ${2} is the file to which 0 will be appended if the run resulted
#			in an incorrect answer, if the answer was correct, we append 1.
#		

#Test whether there is the required golden file present in the archive
GOLDEN_FILES_ZIP=`ls * | grep ${TEST_NAME}.zip`
if test ! -z "${GOLDEN_FILES_ZIP}"; then
	GOLDEN_FILE_IN_ZIP=`eval unzip -l ${TEST_NAME}.zip | grep $1.golden`
	if test ! -z "${GOLDEN_FILE_IN_ZIP}"; then
		#Unpack the golden files
		unzip -u ${TEST_NAME}.zip $1.golden 2>&1 1>/dev/null
	fi
fi

if test -f $1.golden ; then
	#Preprocess the golden file
	#The tests PASS/NOT PASS and the Elapsed time
	cat $1.golden | sed -f ${SIMULATIONS_SED_SCRIPTS}/${CURR_TOOL_NAME}.main.rules > $1.golden.out.tmp
	cat $1.golden.out.tmp | sed -f ${SIMULATIONS_SED_SCRIPTS}/${CURR_TOOL_NAME}.result.rules > $1.golden.tmp
else
	#Just create a dummy 
	echo "ERROR: THE REQUIRED GOLDEN FILE $1.golden WAS NOT FOUND" > $1.golden.tmp
fi

#Perform the check
if diff $1.golden.tmp $1.out > $1.diff ; then
	#Remove the +/- printed by the on_wait_statistics_gathering script and indicate the good run
	echo -ne "\b+"
	#A successfull run
	echo "1" >> ${2}
	#If it is a pass then we do not need the output file for creating the golden out of it.
	rm -f ${TEST_CASE_NAME}.output
else
	#Remove the +/- printed by the on_wait_statistics_gathering script and indicate the bad run
	echo -ne "\b-"
	#An unsuccessfull run
	echo "0" >> ${2}
fi

#Remove temp files
rm -f $1.golden $1.golden.tmp $1.golden.out.tmp $1.diff
