#!/bin/bash -u

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: process_test_case_results.sh,v $
#	$Revision: 1.14 $
#	$Date: 2008/06/09 09:29:56 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2008.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: This script runs simulation tests and is responcible for processing one test case run results.
#

#The directory with scripts responcible for extracting the raw statistical data from test runs
SIMULATION_EXTRACT_STAT_SCRIPTS="${SIMULATIONS_SHELL_SCRIPTS}/extract_data"

#Test if the test run ended with a time out.
if test ${1} -eq 1; then #If YES, Produce the dummy statistics for the used tool
	
	#The time statistics is updated with the time out value
	echo "${TIME_OUT_MILLISEC}" >> ${INPUT_STAT_DIR_NAME}/${TEST_CASE_NAME}.timestat
	#The verification PASS/NOT PASS statistics is updated with the NOT PASS value
	echo "0" >> ${INPUT_STAT_DIR_NAME}/${TEST_CASE_NAME}.confstat
	#The "number of sampled states" statistics is updated with the zero value
	echo "0" >> ${INPUT_STAT_DIR_NAME}/${TEST_CASE_NAME}.samplestat
	
else #If NO, Process the currently gathered data, i.e.
	
	#Remove the temporary files
	rm -f ${TEST_CASE_NAME}.output ${TEST_CASE_NAME}.out
	
	#The tests PASS/NOT PASS and the Elapsed time
	cat ${TEST_CASE_NAME}.out.tmp | sed -f ${SIMULATIONS_SED_SCRIPTS}/${CURR_TOOL_NAME}.main.rules > ${TEST_CASE_NAME}.output
	
	#Update the time statistics file, note that VESTA and Ymer report the model-chekcing time in seconds
	${SIMULATION_EXTRACT_STAT_SCRIPTS}/${CURR_TOOL_NAME}.extract.timestat.sh

	#Extract the "number of samples states" statistics
	${SIMULATION_EXTRACT_STAT_SCRIPTS}/${CURR_TOOL_NAME}.extract.samplestat.sh
	
	#Extract the verification result for comparison with the golden files
	cat ${TEST_CASE_NAME}.output | sed -f ${SIMULATIONS_SED_SCRIPTS}/${CURR_TOOL_NAME}.result.rules > ${TEST_CASE_NAME}.out
	#Update the verification result PASS/NOT PASS statistics in the .verstat file
	#If it is a PASS then the .output file is removed
	${SIMULATIONS_SHELL_SCRIPTS}/pf.sh ${TEST_CASE_NAME} ${INPUT_STAT_DIR_NAME}/${TEST_CASE_NAME}.confstat
	
	#Remove the temporary files
	rm -f ${TEST_CASE_NAME}.out
	
fi
