#!/bin/bash

#
#	WARNING: Do Not Remove This Section
#
#	$RCSfile: stop.sh,v $
#	$Revision: 1.9 $
#	$Date: 2008/06/09 12:40:32 $
#	$Author: i.zapreev $
#	
#	MRMC is a model checker for discrete-time and continuous-time Markov reward models.
#	It supports reward extensions of PCTL and CSL (PRCTL and CSRL), and allows for the
#	automated verification of properties concerning long-run and instantaneous rewards
#	as well as cumulative rewards.
#	
#	Copyright (C) The University of Twente, 2004-2006.
#	Authors: Ivan Zapreev
#
#	This program is free software; you can redistribute it and/or
#	modify it under the terms of the GNU General Public License
#	as published by the Free Software Foundation; either version 2
#	of the License, or (at your option) any later version.
#
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#
#	You should have received a copy of the GNU General Public License
#	along with this program; if not, write to the Free Software
#	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#
#	Our contacts:
#		Formal Methods and Tools Group, University of Twente,
#		P.O. Box 217, 7500 AE Enschede, The Netherlands,
#		Phone: +31 53 4893767, Fax: +31 53 4893247,
#		E-mail: mrmc@cs.utwente.nl
#
#	Source description: Is used to stop test, especially the performance
#	tests where MRMC/Ymer is running in the background mode.
#

TEST_ALL_SH="test_all.sh"
TEST_SH="test.sh"
PERFORMANCE_TEST_SH="test_suite.sh lumping.sh simulation.sh"
MRMC_BIN="mrmc"
YMER_BIN="ymer"
JAVA_BIN="java"
PRISM_BIN="prism"

ALL_EXECUTABLES="${TEST_ALL_SH} ${TEST_SH} ${PERFORMANCE_TEST_SH} ${MRMC_BIN} ${YMER_BIN} ${PRISM_BIN}"

stop_process () {
	let "COUNTER=${COUNTER}+1";
	PROCESS_PID=`eval pidof -x $1`
	if test ! -z "${PROCESS_PID}"; then
		echo "${COUNTER}. Killing the $2, PID:" ${PROCESS_PID}
		kill -9 ${PROCESS_PID}
	else
		echo "${COUNTER}. The $2 is/are not running"
	fi
}

is_there_something_else (){
	#Reset the counter for the stop_process function
	COUNTER="0"
	let "ITERATIONS_COUNTER=${ITERATIONS_COUNTER}+1";
	
	echo -n "*  Iteration ${ITERATIONS_COUNTER}: "
	PROCESS_PID=`eval pidof -x $1`
	if test ! -z "${PROCESS_PID}"; then
		echo "Some unstopped processes detected."
		return 1
	else
		echo "Everything is stopped."
		return 0
	fi
}

echo "+++++++++++++++++ Stopping tests +++++++++++++++"

#Do test for the first iteration
is_there_something_else "${ALL_EXECUTABLES}"
while test "$?" -eq "1"; do
	#Try to stop all processes
	stop_process ${TEST_ALL_SH} "main script";
	stop_process ${TEST_SH} "test scripts";
	stop_process "${PERFORMANCE_TEST_SH}" "performance test scripts";
	stop_process ${MRMC_BIN} "MRMC processes";
	stop_process ${YMER_BIN} "YMER processes";
	stop_process ${JAVA_BIN} "JAVA processes";
	stop_process ${PRISM_BIN} "PRISM processes";
	
	#Just for better output
	echo " "
	
	#Sleep, just in case there was something starting
	sleep 1
	
	#Do test for the next iteration
	is_there_something_else "${ALL_EXECUTABLES}"
done

echo "+++++++++++++++++++++ Done +++++++++++++++++++++"
